unit Wizard;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, comctrls,
  buttons, Wizard_Types;

type

  TWizard = class(TComponent)
  private
    fNext: TBitBtn;
    fPrior: TBitBtn;
    fPage: TPageControl;
    fFinish: TBitBtn;
    fFinishClick: TNotifyEvent;
    fBeforePrior: TWizardBeforeEvent;
    fAfterNext: TWizardAfterEvent;
    fAfterPrior: TWizardAfterEvent;
    fBeforeNext: TWizardBeforeEvent;
    fCounter: Integer;
    fOnCreate: TNotifyEvent;
    fOnDestroy: TNotifyEvent;
    fDesigner: boolean;
    fRoutes: TStringList;
    fCurrRoute: TStringList;
    fDefImg: boolean;
    fJumpNext: boolean;
    fJumpBack: boolean;
    fLast: Integer;
    fFirst: Integer;
    //    fDefRoute: Integer;
    { Private declarations }
    procedure PriorClick(Sender: TObject);
    procedure NextClick(Sender: TObject);
    function GetFinish: TBitBtn;
    function GetNext: TBitBtn;
    function GetPage: TPageControl;
    function GetPrior: TBitBtn;
    procedure SetFinish(const Value: TBitBtn);
    procedure SetNext(const Value: TBitBtn);
    procedure SetPage(const Value: TPageControl);
    procedure SetPrior(const Value: TBitBtn);
    procedure SetDesigner(const Value: boolean);
    procedure SetDefimg(const Value: boolean);
    procedure SetFirst(const Value: Integer);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property CanJumpNext: boolean read fJumpNext write fJumpNext;
    property CanJumpBack: boolean read fJumpBack write fJumpBack;
  published
    { Published declarations }
    property DefImages: boolean read fDefImg write SetDefimg;
    property Prior: TBitBtn read GetPrior write SetPrior;
    property Next: TBitBtn read GetNext write SetNext;
    property Finish: TBitBtn read GetFinish write SetFinish;
    property PageCtrl: TPageControl read GetPage write SetPage;
    property FirstIndex: Integer read fFirst write SetFirst;
    property LastIndex: Integer read fLast write fLast;
    property CurrIndex: Integer read fCounter;
    property OnFinishClick: TNotifyEvent read fFinishClick write fFinishClick;
    property BeforeNext: TWizardBeforeEvent read fBeforeNext write fBeforeNext;
    property AfterNext: TWizardAfterEvent read fAfterNext write fAfterNext;
    property BeforePrior: TWizardBeforeEvent read fBeforePrior write fBeforePrior;
    property AfterPrior: TWizardAfterEvent read fAfterPrior write fAfterPrior;
    property Designer: boolean read fDesigner write SetDesigner;
    property OnCreate: TNotifyEvent read fOnCreate write fOnCreate;
    property OnDestroy: TNotifyEvent read fOnDestroy write fOnDestroy;
  end;

procedure Register;
  {$R BtnGlph.RES}

implementation

procedure Register;
begin
  RegisterComponents('ALCOSoft', [TWizard]);
end;

{ TWizard }


constructor TWizard.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if (AOwner is TForm) then 
  begin
    (AOwner as TForm).BorderStyle := bsDialog;
    (AOwner as TForm).Position := poScreenCenter;
  end;
  if Assigned(fOnCreate) then fOnCreate(Self);
  fRoutes := TStringList.Create;
  fCurrRoute := TStringList.Create;
  Designer := False;
  fFirst := 0;
  fLast := -1;
end;

destructor TWizard.Destroy;
begin
  fCurrRoute.Free;
  fRoutes.Free;
  if Assigned(fOnDestroy) then fOnDestroy(Self);
  inherited;
end;


function TWizard.GetFinish: TBitBtn;
begin
  if Assigned(fFinish) then Result := fFinish
  else 
    Result := nil;
end;

function TWizard.GetNext: TBitBtn;
begin
  if Assigned(fNext) then Result := fNext
  else 
    Result := nil;
end;

function TWizard.GetPage: TPageControl;
begin
  if Assigned(fPage) then Result := fPage
  else 
    Result := nil;
end;

function TWizard.GetPrior: TBitBtn;
begin
  if Assigned(fPrior) then Result := fPrior
  else 
    Result := nil;
end;

procedure TWizard.NextClick(Sender: TObject);
begin
  if not Assigned(fPage) then exit;
  if Assigned(fBeforeNext) then fBeforeNext(Self, fPage.ActivePageIndex, fJumpNext);
  if fLast = -1 then 
  begin
    if CanJumpNext and (fCounter < fPage.PageCount - 1) then 
    begin
      Inc(fCounter);
      fPage.ActivePageIndex := fCounter;
      if Assigned(fAfterNext) then fAfterNext(Self, fPage.ActivePageIndex);
    end;
  end
  else if CanJumpNext and (fCounter < fLast) then 
  begin
    Inc(fCounter);
    fPage.ActivePageIndex := fCounter;
    if Assigned(fAfterNext) then fAfterNext(Self, fPage.ActivePageIndex);
  end;
  CanJumpBack := True;
  if fLast = -1 then
    fFinish.Enabled := (fPage.ActivePageIndex = fPage.PageCount - 1)
  else 
    fFinish.Enabled := (fPage.ActivePageIndex = fLast);
  fNext.Enabled := not fFinish.Enabled;
  fPrior.Enabled := (fPage.ActivePageIndex > fFirst);
end;

procedure TWizard.PriorClick(Sender: TObject);
begin
  if not Assigned(fPage) then exit;
  if Assigned(fBeforePrior) then fBeforePrior(Self, fPage.ActivePageIndex, fJumpBack);
  if CanJumpBack and ((fCounter > fFirst)) then 
  begin
    Dec(fCounter);
    fPage.ActivePageIndex := fCounter;
    if Assigned(fAfterPrior) then fAfterPrior(Self, fPage.ActivePageIndex);
  end;
  canJumpNext := True;
  if fLast = -1 then
    fFinish.Enabled := (fPage.ActivePageIndex = fPage.PageCount - 1)
  else 
    fFinish.Enabled := (fPage.ActivePageIndex = fLast);
  fNext.Enabled := not fFinish.Enabled;
  fPrior.Enabled := (fPage.ActivePageIndex > fFirst);
end;



procedure TWizard.SetDefimg(const Value: boolean);
begin
  fDefImg := Value;
  if Value then 
  begin
    if Assigned(fNext) then 
    begin
      fNext.Glyph.LoadFromResourceName(hInstance, 'NEXT');
      if fNext.Caption = fNext.Name then
        fNext.Caption := 'Next';
    end;
    if Assigned(fFinish) then 
    begin
      fFinish.Glyph.LoadFromResourceName(hInstance, 'FINISH');
      if fFinish.Caption = fFinish.Name then
        fFinish.Caption := 'Finish';
    end;
    if Assigned(fPrior) then 
    begin
      fPrior.Glyph.LoadFromResourceName(hInstance, 'PRIOR');
      if fPrior.Caption = fPrior.Name then
        fPrior.Caption := 'Back';
    end;
  end
  else
  begin
    if Assigned(fNext) then fNext.Glyph := nil;
    if Assigned(fFinish) then fFinish.Glyph := nil;
    if Assigned(fPrior) then fPrior.Glyph := nil;
  end;
end;

procedure TWizard.SetDesigner(const Value: boolean);
var 
  i: integer;
begin
  fDesigner := Value;
  if not Assigned(fPage) then exit;
  if fDesigner then 
  begin
    for i := 0 to fPage.PageCount - 1 do
      fPage.Pages[i].TabVisible := True;
  end
  else 
  begin
    for i := 0 to fPage.PageCount - 1 do
      fPage.Pages[i].TabVisible := False;
    fPage.ActivePageIndex := fFirst;
  end;
end;

procedure TWizard.SetFinish(const Value: TBitBtn);
begin
  if Assigned(Value) then 
  begin
    fFinish := Value;
    with fFinish do 
    begin
      Layout := blGlyphLeft;
      if fDefImg then
        Glyph.LoadFromResourceName(hInstance, 'FINISH');
      OnClick := OnFinishClick;
      Enabled := False;
    end;
  end
  else 
    fFinish := nil;
end;

procedure TWizard.SetFirst(const Value: Integer);
begin
  fFirst := Value;
  if Assigned(fPage) then
    fPage.ActivePageIndex := fFirst;
end;

procedure TWizard.SetNext(const Value: TBitBtn);
begin
  if Assigned(Value) then 
  begin
    fNext := Value;
    fNext.Layout := blGlyphLeft;
    fNext.Default := True;
    if fDefImg then
      fNext.Glyph.LoadFromResourceName(hInstance, 'NEXT');
    fNext.OnClick := NextClick;
  end
  else 
    fNext := nil;
end;

procedure TWizard.SetPage(const Value: TPageControl);
var 
  i: integer;
begin
  if Assigned(Value) then 
  begin
    fPage := Value;
    fPage.ActivePageIndex := fFirst;
    if not Designer then 
    begin
      for i := 0 to fPage.PageCount - 1 do
        fPage.Pages[i].TabVisible := False;
      fPage.ActivePageIndex := fFirst;
    end;
    CanJumpNext := True;
    CanJumpBack := True;
    fCounter := fFirst;
  end
  else 
    fPage := nil;
  //        AfterAssign;
end;

procedure TWizard.SetPrior(const Value: TBitBtn);
begin
  if Assigned(Value) then 
  begin
    fPrior := Value;
    fPrior.Layout := blGlyphLeft;
    if fDefImg then
      fPrior.Glyph.LoadFromResourceName(hInstance, 'PRIOR');
    fPrior.Enabled := False;
    fPrior.OnClick := PriorClick;
  end
  else 
    fPrior := nil;
end;

end.
