// Generated by ParserBuilder @ 19/05/2001 4:25:28 PM

unit WSStripperScanner;

(* NOTE: Any changes made to this file may be overwritten when it gets regenerated
  from within ParserBuilder *)

interface

uses
  Windows, Messages, SysUtils, Classes;

const
  BlkSize = 16384;
  TAB = #09;
  _LF = #10;
  _CR = #13;
  _EF = #0;
  EF = #0;
  _EL = _CR;
  LineEnds: array[0..2] of char = (#13, #10, #0);
  CRLF = #13#10;

type
  TBufBlock = array [0..BlkSize - 1] of char;
  TCocoBuffer = array [0..1024] of ^TBufBlock;
  TStartTable = array [0..255] of Integer;
  TGetCH = procedure(pos: longint; var retCh: char) of object;
  TCocoErrorProc = procedure(nr, line, col: Integer; pos: Integer) of object;

type
  TWSStripperScanner = class(TObject)
  private
    lastCh,
    ch: char;       (*current input character*)
    curLine: Integer;    (*current input line (may be higher than line)*)
    lineStart: longint;    (*start position of current line*)
    apx: longint;    (*length of appendix (CONTEXT phrase)*)
    oldEols: Integer;    (*number of _EOLs in a comment*)
    bp, bp0: longint;    (*current position in buf
                           (bp0: position of current token)*)
    LBlkSize: longint;    (*BlkSize*)
    inputLen: longint;    (*source file size*)
    buf: TCocoBuffer;     (*source buffer for low-level access*)
    start: TStartTable; (*start state for every character*)
    CurrentCh: TGetCH;
    {----------------}
    FnextLen: Integer;
    Fpos: Integer;
    Fcol: Integer;
    FnextCol: Integer;
    FnextPos: Integer;
    Flen: Integer;
    FnextLine: Integer;
    Fline: Integer;
    Fdirectory: string;
    FError: TCocoErrorProc;
    FOutputDirectory: string;
    procedure Err(nr, line, col: Integer; pos: longint);
    procedure NextCh;
    function Comment: boolean;
    procedure CapChAt(pos: longint; var retChar: char);
    procedure DisposeBuf;
  protected
  public
    src: TStringStream;         (*source/list files. To be opened by the main pgm*)
    lst: TStringStream;
    Ferrors: Integer;
    constructor Create;
    destructor Destroy; override;
    procedure Get(var sym: Integer);
    (* Gets next symbol from source file *)
    procedure GetString(pos: longint; len: Integer; var s: string);
    (* Retrieves exact string of max length len from position pos in source file *)
    procedure GetName(pos: longint; len: Integer; var s: string);
    (* Retrieves name of symbol of length len at position pos in source file *)
    procedure CharAt(pos: longint; var retChar: char);
    (* Returns exact character at position pos in source file *)
    procedure _Reset;
    (* Reads and stores source file internally *)
    property Error: TCocoErrorProc read FError write FError;
  published
    { Published declarations }
    property directory: string read Fdirectory write Fdirectory;
    (*of source file*)
    property OutputDirectory: string read FOutputDirectory write FOutputDirectory;
    property line: Integer read Fline write Fline;
    (*line and column of current symbol*)
    property col: Integer read Fcol write Fcol;
    (*line and column of current symbol*)
    property len: Integer read Flen write Flen;      (*length of current symbol*)
    property pos: Integer read Fpos write Fpos;
    (*file position of current symbol*)
    property nextLine: Integer read FnextLine write FnextLine;
    (*line of lookahead symbol*)
    property nextCol: Integer read FnextCol write FnextCol;
    (*column of lookahead symbol*)
    property nextLen: Integer read FnextLen write FnextLen;
    (*length of lookahead symbol*)
    property nextPos: Integer read FnextPos write FnextPos;
    (*file position of lookahead symbol*)
    property errors: Integer read Ferrors write Ferrors;
    (*number of detected errors*)
  end;


implementation



const
  no_Sym = 4; (*error token code*)
  (* not only for errors but also for not finished states of scanner analysis *)
  _EOF = #26; (*MS-DOS _EOF*)

procedure TWSStripperScanner.CapChAt(pos: longint; var retChar: char);
var
  ch: char;
begin
  if pos >= inputLen then
  begin
    retChar := EF;
    EXIT;
  end;
  ch := upcase(buf[pos div LBlkSize]^[pos mod LBlkSize]);
  if ch <> _EOF then retChar := ch
  else
    retChar := EF
end;

procedure TWSStripperScanner.CharAt(pos: longint; var retChar: char);
var
  ch: char;
begin
  if pos >= inputLen then
  begin
    retChar := EF;
    EXIT;
  end;
  ch := buf[pos div LBlkSize]^[pos mod LBlkSize];
  if ch <> _EOF then retChar := ch
  else
    retChar := EF
end;

function TWSStripperScanner.Comment: boolean;
begin
  Comment := False;
end;


procedure TWSStripperScanner.Err(nr, line, col, pos: Integer);
begin
  INC(Ferrors)
end;

procedure TWSStripperScanner.Get(var sym: Integer);
var
  state: integer;

  function Equal(s: string): boolean;
  var
    i: integer;
    q: longint;
    cc: char; // Current Char returned from modifed TGetCh
  begin
    if nextLen <> Length(s) then
    begin
      Equal := False;
      EXIT
    end;
    i := 1;
    q := bp0;
    while i <= nextLen do
    begin
      CurrentCh(q, cc);
      if cc <> s[i] then
      begin
        Equal := False;
        EXIT
      end;
      INC(i);
      INC(q)
    end;
    Equal := True
  end;
begin (*Get*)
  while (ch = ' ') or
        
    ((ch >= CHR(1)) and (ch <= CHR(31))) do NextCh;
  pos := nextPos;
  nextPos := bp;
  col := nextCol;
  nextCol := bp - lineStart;
  line := nextLine;
  nextLine := curLine;
  len := nextLen;
  nextLen := 0;
  apx := 0;
  state := start[Ord(ch)];
  bp0 := bp;
  while True do
  begin
    NextCh;
    INC(FnextLen);
    case state of
      1: if ((ch >= '0') and (ch <= '9') or
             
          (ch >= 'A') and (ch <= 'Z') or
             
          (ch = '_')) then 
        begin            
        end 
        else 
        begin 
          sym := 1; 
          EXIT; 
        end;
     
      2: if ((ch = CHR(0)) or
             
          (ch >= ' ') and (ch <= '!') or
             
          (ch >= '#')) then 
        begin            
        end 
        else if (ch = '"') then 
        begin 
          state := 4; 
        end 
        else 
        begin 
          sym := no_Sym; 
          EXIT; 
        end;
     
      3: if ((ch = CHR(0)) or
             
          (ch >= ' ') and (ch <= '&') or
             
          (ch >= '(')) then 
        begin            
        end 
        else if (ch = CHR(39)) then 
        begin 
          state := 4; 
        end 
        else 
        begin 
          sym := no_Sym; 
          EXIT; 
        end;
     
      4: 
      begin 
        sym := 2; 
        EXIT; 
      end;
     
      5: 
      begin 
        sym := 3; 
        EXIT; 
      end;
     
      6: 
      begin 
        sym := 0; 
        ch := #0; 
        DEC(bp); 
        EXIT 
      end;
      else
      begin
        sym := no_Sym;
        EXIT (*NextCh already done*)
      end;
    end
  end
end;


procedure TWSStripperScanner.GetName(pos, len: Integer; var s: string);
var
  i: integer;
  p: longint;
  cc: char;
begin
  if len > 255 then len := 255;
  p := pos;
  i := 1;
  SetLength(S, Len);
  while i <= len do
  begin
    CurrentCh(p, cc);
    s[i] := cc;
    INC(i);
    INC(p)
  end;
end;


procedure TWSStripperScanner.GetString(pos, len: Integer; var s: string);
var
  i: integer;
  p: longint;
  cc: char;
begin
  if len > 255 then len := 255;
  p := pos;
  i := 1;
  setLength(S, len);
  while i <= len do
  begin
    CharAt(p, cc);
    s[i] := cc;
    INC(i);
    INC(p)
  end;
end;

procedure TWSStripperScanner.NextCh;
  (* Return global variable ch *)
var 
  cc: char;
begin
  lastCh := ch;
  INC(bp);
  CurrentCh(bp, cc);
  ch := cc;
  if (ch = _EL) or (ch = _LF) and (lastCh <> _EL) then
  begin
    INC(curLine);
    lineStart := bp
  end
end;


procedure TWSStripperScanner._Reset;
var
  len: longint;
  i, Read: integer;
  AmtTransferred: integer;
begin (*assert: src has been opened*)
  //  len := FileSize(src);
  Len := src.Size;
  DisposeBuf;
  i := 0;
  inputLen := len;
  while len > LBlkSize do
  begin
    NEW(buf[i]);
    Read := BlkSize;
    //    BlockRead(src, buf[i]^, Read,AmtTransferred);
    AmtTransferred := src.Read(buf[i]^, Read);
    len := len - AmtTransferred;
    INC(i)
  end;
  NEW(buf[i]);
  Read := len;
  //AmtTransferred :=
  src.Read(buf[i]^, Read);
  buf[i]^[Read] := EF;
  curLine := 1;
  lineStart := -2;
  bp := -1;
  oldEols := 0;
  apx := 0;
  errors := 0;
  NextCh;
end;

constructor TWSStripperScanner.Create;
begin
  inherited;
  lst := TStringStream.Create('');
  src := nil;
  CurrentCh := CapChAt;
  
  start[0] := 6; 
  start[1] := 7; 
  start[2] := 7; 
  start[3] := 7; 
  
  start[4] := 7; 
  start[5] := 7; 
  start[6] := 7; 
  start[7] := 7; 
  
  start[8] := 7; 
  start[9] := 7; 
  start[10] := 7; 
  start[11] := 7; 
  
  start[12] := 7; 
  start[13] := 7; 
  start[14] := 7; 
  start[15] := 7; 
  
  start[16] := 7; 
  start[17] := 7; 
  start[18] := 7; 
  start[19] := 7; 
  
  start[20] := 7; 
  start[21] := 7; 
  start[22] := 7; 
  start[23] := 7; 
  
  start[24] := 7; 
  start[25] := 7; 
  start[26] := 7; 
  start[27] := 7; 
  
  start[28] := 7; 
  start[29] := 7; 
  start[30] := 7; 
  start[31] := 7; 
  
  start[32] := 7; 
  start[33] := 7; 
  start[34] := 2; 
  start[35] := 7; 
  
  start[36] := 7; 
  start[37] := 5; 
  start[38] := 7; 
  start[39] := 3; 
  
  start[40] := 5; 
  start[41] := 5; 
  start[42] := 5; 
  start[43] := 7; 
  
  start[44] := 5; 
  start[45] := 5; 
  start[46] := 5; 
  start[47] := 5; 
  
  start[48] := 1; 
  start[49] := 1; 
  start[50] := 1; 
  start[51] := 1; 
  
  start[52] := 1; 
  start[53] := 1; 
  start[54] := 1; 
  start[55] := 1; 
  
  start[56] := 1; 
  start[57] := 1; 
  start[58] := 5; 
  start[59] := 5; 
  
  start[60] := 7; 
  start[61] := 7; 
  start[62] := 7; 
  start[63] := 7; 
  
  start[64] := 5; 
  start[65] := 1; 
  start[66] := 1; 
  start[67] := 1; 
  
  start[68] := 1; 
  start[69] := 1; 
  start[70] := 1; 
  start[71] := 1; 
  
  start[72] := 1; 
  start[73] := 1; 
  start[74] := 1; 
  start[75] := 1; 
  
  start[76] := 1; 
  start[77] := 1; 
  start[78] := 1; 
  start[79] := 1; 
  
  start[80] := 1; 
  start[81] := 1; 
  start[82] := 1; 
  start[83] := 1; 
  
  start[84] := 1; 
  start[85] := 1; 
  start[86] := 1; 
  start[87] := 1; 
  
  start[88] := 1; 
  start[89] := 1; 
  start[90] := 1; 
  start[91] := 7; 
  
  start[92] := 7; 
  start[93] := 7; 
  start[94] := 7; 
  start[95] := 1; 
  
  start[96] := 7; 
  start[97] := 7; 
  start[98] := 7; 
  start[99] := 7; 
  
  start[100] := 7; 
  start[101] := 7; 
  start[102] := 7; 
  start[103] := 7; 
  
  start[104] := 7; 
  start[105] := 7; 
  start[106] := 7; 
  start[107] := 7; 
  
  start[108] := 7; 
  start[109] := 7; 
  start[110] := 7; 
  start[111] := 7; 
  
  start[112] := 7; 
  start[113] := 7; 
  start[114] := 7; 
  start[115] := 7; 
  
  start[116] := 7; 
  start[117] := 7; 
  start[118] := 7; 
  start[119] := 7; 
  
  start[120] := 7; 
  start[121] := 7; 
  start[122] := 7; 
  start[123] := 5; 
  
  start[124] := 7; 
  start[125] := 5; 
  start[126] := 7; 
  start[127] := 7; 
  
  start[128] := 7; 
  start[129] := 7; 
  start[130] := 7; 
  start[131] := 7; 
  
  start[132] := 7; 
  start[133] := 7; 
  start[134] := 7; 
  start[135] := 7; 
  
  start[136] := 7; 
  start[137] := 7; 
  start[138] := 7; 
  start[139] := 7; 
  
  start[140] := 7; 
  start[141] := 7; 
  start[142] := 7; 
  start[143] := 7; 
  
  start[144] := 7; 
  start[145] := 7; 
  start[146] := 7; 
  start[147] := 7; 
  
  start[148] := 7; 
  start[149] := 7; 
  start[150] := 7; 
  start[151] := 7; 
  
  start[152] := 7; 
  start[153] := 7; 
  start[154] := 7; 
  start[155] := 7; 
  
  start[156] := 7; 
  start[157] := 7; 
  start[158] := 7; 
  start[159] := 7; 
  
  start[160] := 7; 
  start[161] := 7; 
  start[162] := 7; 
  start[163] := 7; 
  
  start[164] := 7; 
  start[165] := 7; 
  start[166] := 7; 
  start[167] := 7; 
  
  start[168] := 7; 
  start[169] := 7; 
  start[170] := 7; 
  start[171] := 7; 
  
  start[172] := 7; 
  start[173] := 7; 
  start[174] := 7; 
  start[175] := 7; 
  
  start[176] := 7; 
  start[177] := 7; 
  start[178] := 7; 
  start[179] := 7; 
  
  start[180] := 7; 
  start[181] := 7; 
  start[182] := 7; 
  start[183] := 7; 
  
  start[184] := 7; 
  start[185] := 7; 
  start[186] := 7; 
  start[187] := 7; 
  
  start[188] := 7; 
  start[189] := 7; 
  start[190] := 7; 
  start[191] := 7; 
  
  start[192] := 7; 
  start[193] := 7; 
  start[194] := 7; 
  start[195] := 7; 
  
  start[196] := 7; 
  start[197] := 7; 
  start[198] := 7; 
  start[199] := 7; 
  
  start[200] := 7; 
  start[201] := 7; 
  start[202] := 7; 
  start[203] := 7; 
  
  start[204] := 7; 
  start[205] := 7; 
  start[206] := 7; 
  start[207] := 7; 
  
  start[208] := 7; 
  start[209] := 7; 
  start[210] := 7; 
  start[211] := 7; 
  
  start[212] := 7; 
  start[213] := 7; 
  start[214] := 7; 
  start[215] := 7; 
  
  start[216] := 7; 
  start[217] := 7; 
  start[218] := 7; 
  start[219] := 7; 
  
  start[220] := 7; 
  start[221] := 7; 
  start[222] := 7; 
  start[223] := 7; 
  
  start[224] := 7; 
  start[225] := 7; 
  start[226] := 7; 
  start[227] := 7; 
  
  start[228] := 7; 
  start[229] := 7; 
  start[230] := 7; 
  start[231] := 7; 
  
  start[232] := 7; 
  start[233] := 7; 
  start[234] := 7; 
  start[235] := 7; 
  
  start[236] := 7; 
  start[237] := 7; 
  start[238] := 7; 
  start[239] := 7; 
  
  start[240] := 7; 
  start[241] := 7; 
  start[242] := 7; 
  start[243] := 7; 
  
  start[244] := 7; 
  start[245] := 7; 
  start[246] := 7; 
  start[247] := 7; 
  
  start[248] := 7; 
  start[249] := 7; 
  start[250] := 7; 
  start[251] := 7; 
  
  start[252] := 7; 
  start[253] := 7; 
  start[254] := 7; 
  start[255] := 7; 
  Error := Err;
  LBlkSize := BlkSize;
  lastCh := EF;
end;

destructor TWSStripperScanner.Destroy;
begin
  if (src <> nil) then
    src.Free;
  lst.Free;
  DisposeBuf;
  inherited;
end;

procedure TWSStripperScanner.DisposeBuf;
var
  idx: integer;
begin
  for idx := 1024 downto 0 do
  begin
    if buf[idx] <> nil then
      Dispose(buf[idx]);
    buf[idx] := nil;
  end;
end;



end.
