{-----------------------------------------------------------------------------
 Unit Name: CocoFactor
 Author:    Administrator
 Purpose: Container for GUI-model of ATG File - Holds the factors
-----------------------------------------------------------------------------}


unit CocoFactor;

interface

uses
  Windows, Messages, SysUtils, Classes, pbStrLib, Dialogs, SemActionFormatterComponent;

type
  TCocoFactor = class(TComponent)
  private
    { Private declarations }
    FAttributes: string;
    FSymbol: string;
    FExpression: TComponent;
    FIsWeak: boolean;
    FSemanticText: string;
    FIsANY: boolean;
    FIsSYNC: boolean;
    FInsideBrace: boolean;
    FIsNoneOrMany: boolean;
    FIsNoneOrOne: boolean;
    FIsBranching: boolean;
    FDoesCloseNoneOrOne: boolean;
    FDoesCloseNoneOrMany: boolean;
    FDoesCloseInsideBrace: boolean;
    FOpenPunctuation: string;
    FClosePunctuation: string;
    FIndent: Integer;
    FComments: string;
    fsaMinIndent: Integer;
    procedure SetAttributes(const Value: string);
    procedure SetSymbol(const Value: string);
    procedure SetExpression(const Value: TComponent);
    procedure SetIsWeak(const Value: boolean);
    procedure SetSemanticText(const Value: string);
    procedure SetIsANY(const Value: boolean);
    procedure SetIsSYNC(const Value: boolean);
    procedure SetInsideBrace(const Value: boolean);
    procedure SetIsNoneOrMany(const Value: boolean);
    procedure SetIsNoneOrOne(const Value: boolean);
    function GetSemanticWhiteSpace(const LineText: string;
      const iStart: Integer): string;
    procedure SetIsBranching(const Value: boolean);
    procedure SetDoesCloseInsideBrace(const Value: boolean);
    procedure SetDoesCloseNoneOrMany(const Value: boolean);
    procedure SetDoesCloseNoneOrOne(const Value: boolean);
    procedure SetClosePunctuation(const Value: string);
    procedure SetOpenPunctuation(const Value: string);
    procedure SetIndent(const Value: Integer);
    procedure SetComments(const Value: string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner: TComponent); override;
    destructor Destroy; override;
    function GetATG: string;
  published
    { Published declarations }
    property Symbol: string read FSymbol write SetSymbol;
    property Attributes: string read FAttributes write SetAttributes;
    property Expression: TComponent read FExpression write SetExpression;
    property IsWeak: boolean read FIsWeak write SetIsWeak;
    property SemanticText: string read FSemanticText write SetSemanticText;
    property IsANY: boolean read FIsANY write SetIsANY;
    property IsSYNC: boolean read FIsSYNC write SetIsSYNC;
    property IsNoneOrMany: boolean read FIsNoneOrMany write SetIsNoneOrMany;
    property IsNoneOrOne: boolean read FIsNoneOrOne write SetIsNoneOrOne;
    property InsideBrace: boolean read FInsideBrace write SetInsideBrace;
    property IsBranching: boolean read FIsBranching write SetIsBranching;
    property Indent: Integer read FIndent write SetIndent;

    // New - Indent properties for semantic actions
    property saMinIndent: Integer read fsaMinIndent write fsaMinIndent;

    property DoesCloseNoneOrMany: boolean read FDoesCloseNoneOrMany          
      write SetDoesCloseNoneOrMany;
    property DoesCloseNoneOrOne: boolean read FDoesCloseNoneOrOne write SetDoesCloseNoneOrOne;
    property DoesCloseInsideBrace: boolean read FDoesCloseInsideBrace          
      write SetDoesCloseInsideBrace;
    property OpenPunctuation: string read FOpenPunctuation write SetOpenPunctuation;
    property ClosePunctuation: string read FClosePunctuation write SetClosePunctuation;
    property Comments: string read FComments write SetComments;
  end;

implementation

uses CocoProductionPath;

{ TCocoFactor }

constructor TCocoFactor.Create(Aowner: TComponent);
begin
  inherited;
  FExpression := nil; // this value gets assigned by the parser
  fsaMinIndent := 40; // default indent for semantic actions
end;

destructor TCocoFactor.Destroy;
begin
  if (fExpression <> nil) then
    fExpression.Free;
  inherited;
end;

function TCocoFactor.GetATG: string;
var
  S: string;
  iStart: integer;
begin
  //  iStart := 0;
  S := '';
  // The code here is useless
  // but it makes it easier to set
  // a breakpoint without mucking with bp conditions. The bracketting is fucking up :-(
  //  if (fOpenPunctuation <> '') then
  if (Pos('{',fOpenPunctuation)>0) or (Pos('[',fOpenPunctuation)>0)  then
  Begin
       S := CRLF+fOpenPunctuation;
  End
  else
    S := fOpenPunctuation;
  //  else
  //    S := '';
//  if (fIsWeak) and (FExpression = nil) and (FOpenPunctuation = '') then S := S + 'WEAK ';
  if (fIsWeak) and (Symbol <> 'WEAK')  then
  Begin // made a block for debugging
  S := S + 'WEAK ';
  End;
  if (fIsANY) then S := S + 'ANY '
  else if (fIsSYNC) then S := S + 'SYNC '
  else
    S := S + Symbol + ' ';
  if (FIsBranching) then S := '| ' + S;
  if (fAttributes <> '') then
    S := S + '< ' + trim(fAttributes) + ' > ';
  if (FExpression <> nil) then
  begin
    Assert(fExpression is TCocoProductionPath,
      'Wrong object type assigned to fExpression.');
    TCocoProductionPath(fExpression).saMinIndent := fsaMinIndent;
    S := S + Trim(TCocoProductionPath(fExpression).GetATG);
  end;

  iStart := StrColumns(S, #13#10);
  iStart := Length(StrExtract(S, iStart, #13#10));
  if (iStart > 1) then iStart := 12;

  if (fSemanticText <> '') then
    Result := S + GetSemanticWhiteSpace(S, iStart) + #13#10
  else
    Result := S;

  if (FClosePunctuation <> '') then
    Result := Result + ' ' + FClosePunctuation+ #13#10;
end;

function TCocoFactor.GetSemanticWhiteSpace(const LineText: string;
  const iStart: Integer): string;
var 
  S: string;
begin
  if (Trim(FSemanticText) = '') then
    Result := ''
  else
  begin
    if FSemanticText[1] = '.' then system.Delete(FSemanticText, 1, 1);
    ExecuteSemActionFormatterParser('(. ' + FSemanticText + '.)', fsaMinIndent, iStart, S);
    Result := S
  end;
end;

//function TCocoFactor.GetSemanticWhiteSpace(const LineText: string;
//  const iStart: Integer): string;
//var
//  Rows: integer;
//  S, S1, SOC: string;
//  idx, InitialOffset: integer;
//  // being lazy? - using a string list to break the semantic text into lines
//  // and to provide some manner of pascal source formatting - just to make the ATG
//  // file a little bit cleaner.
//  SL: TStringList;
//begin
//  Result := '';
//  if (FSemanticText <> '') then
//  begin
//    S := fSemanticText;
//    if (Length(LineTExt) > 0) then
//      InitialOffset := 57 - iStart - Length(LineText)
//    else
//      InitialOffset := 57 - iStart;
//    SOC := StringOfChar(' ', InitialOffset);
//
//    SL := TStringList.Create;
//    S := ReplaceAll(S, ';', ';'+#13#10);
//    StringToStringList(S, #13#10, SL);
//    for idx := Sl.Count - 1 downto 0 do
//    begin
//      SL[idx] := Trim(SL[idx]);
//      if (SL[idx] = '') then SL.Delete(idx)
//      else
//      begin
//        if (idx > 0) then
//          SL[idx] := StringOfChar(' ', 60) + SL[idx];
//      end;
//    end;
//    Result := SOC + '(. ' + Trim(SL.Text) + ' .)';
//    SL.Free;
//  end;
//  Exit;
//  // Previous implementation below -->
//  //  if (FSemanticText <> '') then
//  //  begin
//  //    S := fSemanticText;
//  //    if (Length(LineTExt)>0) then
//  //    InitialOffset := 57 - iStart-Length(LineText)+3
//  //    else
//  //     InitialOffset := 57 - iStart;
//  //
//  //    SOC := StringOfChar(' ', InitialOffset);
//  //    Result := SOC + '(. ';
//  //    Rows := StrColumns(S, #13#10);
//  //    for idx := 1 to Rows do
//  //    begin
//  //      if (idx < Rows) then
//  //        S1 := StrExtract(S, idx, #13#10) + #13#10
//  //      else
//  //        S1 := StrExtract(S, idx, #13#10);
//  //
//  //      if Trim(S1) <> '' then
//  //      begin
//  //        if (idx > 1) and (pos(SOC, S1) > 1) then
//  //
//  //          S1 := #13#10 + SOC + '   ' + S1;
//  //        if (Trim(S1) <> '') then
//  //          Result := Result + S1;
//  //      end;
//  //    end;
//  //    Result := Result + ' .)'; // #13#10+SOC
//  //  end
//  //  else
//  //    Result := '';
//end;

procedure TCocoFactor.SetAttributes(const Value: string);
begin
  FAttributes := Value;
end;

procedure TCocoFactor.SetClosePunctuation(const Value: string);
begin
  FClosePunctuation := Value;
end;

procedure TCocoFactor.SetComments(const Value: string);
begin
  FComments := Value;
end;

procedure TCocoFactor.SetDoesCloseInsideBrace(const Value: boolean);
begin
  FDoesCloseInsideBrace := Value;
end;

procedure TCocoFactor.SetDoesCloseNoneOrMany(const Value: boolean);
begin
  FDoesCloseNoneOrMany := Value;
end;

procedure TCocoFactor.SetDoesCloseNoneOrOne(const Value: boolean);
begin
  FDoesCloseNoneOrOne := Value;
end;

procedure TCocoFactor.SetExpression(const Value: TComponent);
begin
  Assert(Value is TCocoProductionPath, 'Bad Expression Assignment.');
  FExpression := Value;
end;

procedure TCocoFactor.SetIndent(const Value: Integer);
begin
  FIndent := Value;
end;

procedure TCocoFactor.SetInsideBrace(const Value: boolean);
begin
  FInsideBrace := Value;
end;

procedure TCocoFactor.SetIsANY(const Value: boolean);
begin
  FIsANY := Value;
end;

procedure TCocoFactor.SetIsBranching(const Value: boolean);
begin
  FIsBranching := Value;
end;

procedure TCocoFactor.SetIsNoneOrMany(const Value: boolean);
begin
  FIsNoneOrMany := Value;
end;

procedure TCocoFactor.SetIsNoneOrOne(const Value: boolean);
begin
  FIsNoneOrOne := Value;
end;

procedure TCocoFactor.SetIsSYNC(const Value: boolean);
begin
  FIsSYNC := Value;
end;

procedure TCocoFactor.SetIsWeak(const Value: boolean);
begin
  FIsWeak := Value;
end;

procedure TCocoFactor.SetOpenPunctuation(const Value: string);
begin
  FOpenPunctuation := Value;
end;

procedure TCocoFactor.SetSemanticText(const Value: string);
begin
  FSemanticText := Value;
end;

procedure TCocoFactor.SetSymbol(const Value: string);
begin
  FSymbol := Value;
end;


end.
